
library(ape)
library(phangorn)
library(stringr)


source("C:/Users/GRJ/AAA/Programming/Biology/DENIMSimulations/Utils/scenario-dir-names.R")
source("C:/Users/GRJ/AAA/Programming/Biology/DENIMSimulations/Utils/mcoal-tree-to-Newick.R")





# Adds an edge to a rooted tree at its root. 
# This makes it into an unrooted tree, with one more tip labelled "origin"
add.origin.to.rooted.tree <- function(tree) {
	stopifnot(is.rooted(tree))
	rooti <- getRoot(tree)
	ntips <- length(tree$tip.label)
	utree <- tree
	w <- which(tree$edge > ntips)
	utree$edge[w] <- tree$edge[w] + 1
	utree$edge <- rbind(utree$edge, c(rooti+1, ntips+1))
	utree$edge.length <- c(utree$edge.length, 0)
	utree$tip.label <- c(utree$tip.label, "origin")
	utree$Nnode <- utree$Nnode + 1
	utree
}



rooted.KF.dist <- function(tree1, tree2) {
	KF.dist(add.origin.to.rooted.tree(tree1), add.origin.to.rooted.tree(tree2))
}


tree.topologies.agree <- function(tree1, tree2) {
	(RF.dist(tree1, tree2) == 0)
}


true.phylo <- function(sdir, rep) {
	ctrlfpath <- paste0(beast.dir.from.scenario.dir.and.rep(sdir, rep), "/ctrl.txt")
	newick <- mcctr.nwk.mc.file.to.newick(ctrlfpath)
	read.tree(text=newick)
}




mcmc.treelist <- function(sdir, rep) {
	tree.fpath <- paste0(beast.dir.from.scenario.dir.and.rep(sdir, rep), "/species.trees")
	treelist <- read.nexus(tree.fpath, force.multi = TRUE)
  treelist <- treelist[-(1:burnin.from.Nsamples(length(treelist)))]
}


# Not used(?)
true.treelength <- function(true.phy) {
  nodeheights <- sort(branching.times(true.phy))
  treelength <- sum(nodeheights) + nodeheights[length(nodeheights)]
  treelength
}


# Compares sampled trees to true tree.
# Returns means of KF distances (averaged over samples) 
# for each scenario and replicate.
make.mean.kfdists <- function(scenario.directories) {
	posterior.mean.kfdists <-
		matrix(0,
					 ncol = length(scenario.directories),
					 nrow = number.of.replicates)
	colnames(posterior.mean.kfdists) <- scenario.directories
	
	for (sdir in scenario.directories) {
		cat(sdir, "... ")
		for (rep in first.replicate:last.replicate) {
			true.phy <- true.phylo(sdir, rep)
			treelist <- mcmc.treelist(sdir, rep)
			
			kfdists <- rep(0, length(treelist))
			for (i in 1:length(treelist)) {
				kfdists[i] <- rooted.KF.dist(treelist[[i]], true.phy)
			}

			posterior.mean.kfdists[rep, sdir] <-  mean(kfdists)
		}
		cat("Done mean distance to true tree\n")
	}
	posterior.mean.kfdists
}



# Compares sampled trees to true tree.
# Returns posterior probability of true tree 
# for each scenario and replicate..
posterior.prob.true.tree.topol <- function(scenario.directories) {
	posterior.prob.tree.topol <-
		matrix(0,
					 ncol = length(scenario.directories),
					 nrow = number.of.replicates)
	colnames(posterior.prob.tree.topol) <- scenario.directories
	
	for (sdir in scenario.directories) {
		cat(sdir, "... ")
		for (rep in first.replicate:last.replicate) {
			true.phy <- true.phylo(sdir, rep)
			treelist <- mcmc.treelist(sdir, rep)
			
			ntrees.right.topol <- 0
			for (i in 1:length(treelist)) {
				if (tree.topologies.agree(treelist[[i]], true.phy)) {
					ntrees.right.topol <- ntrees.right.topol + 1
				}
			}
			
			posterior.prob.tree.topol[rep, sdir] <- ntrees.right.topol / length(treelist)
		}
		cat("Done posterior probs of true stree topology\n")
	}
	posterior.prob.tree.topol
}






#######################################################################################



write.posterior.values <- function(posterior.values, sdirs, reorder, fpath) {
	xxx <- posterior.values[, reorder]
	colnames(xxx) <- sdirs[reorder]
	write.table(xxx, fpath, quote=FALSE, row.names = FALSE)
}


plot.boxplot.vertical <- function(posterior.values, title) {
	sdirs <- colnames(posterior.values)
	par(mfcol = c(1, 1))
	par(mar = c(9, 3, 4, 2) + 0.1)
	
	boxplot(posterior.values, xaxt = "n", 
					main = title)
	axis(1,
			 at = 1:length(sdirs),
			 labels = labels.for.plot(sdirs),
			 las = 3)
}




plot.boxplot.horizontal <- function(posterior.values, title) {
	sdirs <- colnames(posterior.values)
	par(mfcol = c(1, 1))
	par(mar = c(3, 8, 4, 0) + 0.1)
	
	boxplot(posterior.values, yaxt = "n", horizontal=TRUE,
					main = title)
	axis(2,
			 at = 1:length(sdirs),
			 labels = labels.for.plot(sdirs),
			 las = 1)
}




plot.boxplot.screen.and.pdf.vertical <- function(posterior.values, fpath, title) {
	plot.boxplot.vertical(posterior.values, title)
	pdf(file=fpath, pointsize=10, width=6, height=8)
	plot.boxplot.vertical(posterior.values, title)
	dev.off()
}



plot.boxplot.screen.and.pdf.horizontal <- function(posterior.values, fpath, title) {
	plot.boxplot.horizontal(posterior.values, title)
	pdf(file=fpath, pointsize=10, width=4.5, height=6)
	plot.boxplot.horizontal(posterior.values, title)
	dev.off()
}



####################################################################################


sdirs.and.plotorder.spp.4 <- function() {
	list(sdirs=fourspp.all.scenario.dirs, reorder=c(13, 1:12, 14:17))
}

	

#################################################################################################


# Chooses ordering depending on which scenarios are included
sdirs.and.plotorder.spp.10 <- function() {
	nnn <- 0
	if (just.coal.paraphyly) { nnn <- nnn + 1 }
	if (just.leache) { nnn <- nnn + 1 }
	if (just.coal.big.rates) { nnn <- nnn + 1 }
	stopifnot(nnn <= 1)
	
	if (just.coal.paraphyly) {
		sdirs <- tenspp.just.coal.paraphyly.dirs
		reorder <- c(5,1,2,3,4,6,7)
	} else if (just.leache) {
		sdirs <- tenspp.leache.scenario.dirs
		reorder <- 1:19
	} else if (just.coal.big.rates) {
		sdirs <- c("COAL005", "COAL01", "COAL02", "COAL04", "COAL08", "COAL16")
		reorder <- 1:6
	} else {
		sdirs <- tenspp.all.scenario.dirs
		reorder <- c(19, 1:18, 20:25)
	}
	list(sdirs=sdirs, reorder=reorder)
}


sdirs.and.plotorder <- function(nspp) {
	if (nspp == 10) {
		xxx <- sdirs.and.plotorder.spp.10()
		}
	if (nspp == 4) {
		xxx <- sdirs.and.plotorder.spp.4()
	}
	xxx
}

####################################################################################################


find.branch.scores.PPts <- function(nspp) {
	xxx <- sdirs.and.plotorder(nspp)

	posterior.mean.kfdists <- make.mean.kfdists(xxx$sdirs)
	write.posterior.values(posterior.mean.kfdists, xxx$sdirs, xxx$reorder, fpath.for.branch.scores.txt())

	posterior.ptrue.topol <- posterior.prob.true.tree.topol(xxx$sdirs)
	write.posterior.values(posterior.ptrue.topol, xxx$sdirs, xxx$reorder, fpath.for.ptrue.topol.txt())
}


plot.branch.scores.PPts <- function(nspp) {
	xxx <- sdirs.and.plotorder(nspp)
	
	# read.table() does not read column names with "-" in them properly, they become ".". 
	# So I replace colnames with xxx$sdirs[xxx$reorder]
	
	pvals <- read.table(fpath.for.branch.scores.txt(), header = TRUE)
	colnames(pvals) <- xxx$sdirs[xxx$reorder]
	plot.boxplot.screen.and.pdf.vertical(pvals, fpath.for.branch.scores.pdf(), 
																			 "Posterior mean of branch scores")
	
	pvals <- read.table(fpath.for.ptrue.topol.txt(), header = TRUE)
	colnames(pvals) <- xxx$sdirs[xxx$reorder]
	plot.boxplot.screen.and.pdf.horizontal(pvals,	fpath.for.ptrue.topol.pdf(),
																				 "Posterior probability of true topology")
}



do.branch.scores.PPts <- function(nspp) {
	find.branch.scores.PPts(nspp)
	plot.branch.scores.PPts(nspp)
}














